﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Threading;
using System.Diagnostics;

namespace AsyncDataBinding
{
    public class SlowThing : INotifyPropertyChanged
    {
        private string dataValue = "Foo";
        public string Data
        {
            get
            {
                Debug.WriteLine("Read thread: " + Thread.CurrentThread.ManagedThreadId);
                return dataValue;
            }
            set
            {
                Debug.WriteLine("Write thread: " + Thread.CurrentThread.ManagedThreadId);
                if (value != dataValue)
                {
                    dataValue = value;
                    OnPropertyChanged("Data");
                }
            }
        }

        public void GetData()
        {
            Debug.WriteLine("GetData thread: " + Thread.CurrentThread.ManagedThreadId);
            ThreadPool.QueueUserWorkItem(delegate
            {
                Debug.WriteLine("Worker thread: " + Thread.CurrentThread.ManagedThreadId);
                Thread.Sleep(5000);
                this.Data += "!**!@";
            });
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            Debug.WriteLine("Property change thread: " + Thread.CurrentThread.ManagedThreadId);
            Delegate[] handlers = null;
            lock (this)
            {
                if (PropertyChanged != null)
                {
                    handlers = PropertyChanged.GetInvocationList();
                }
            }

            if (handlers != null)
            {
                foreach (PropertyChangedEventHandler h in handlers)
                {
                    h(this, new PropertyChangedEventArgs(propertyName));
                }
            }
        }
    }
}
